classdef VUS
    %UNTITLED2 Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
% PDFs of the transformed variables
        f1
        f2
        f3
% CDFs of the transformed variables
        F1
        F2
        F3
% Inverse CDFs of the transformed variables
        F1inv
        F2inv
        F3inv
% Grid for plotting ROC Surface
        tc1gr
        tc2gr
        tc3gr
        c1gr
        c2gr 
        vus
        vusH0
        gr_sz
    end
    
    methods
        function obj = VUS(f1,f2,f3,F1,F2,F3,F1inv,F2inv,F3inv,gr_sz)
            if nargin < 10
                gr_sz = 0.0025;
            end
            obj.gr_sz = gr_sz;
            
            obj.f1 = f1; obj.F1 = F1; obj.F1inv = F1inv;
            obj.f2 = f2; obj.F2 = F2; obj.F2inv = F2inv;
            obj.f3 = f3; obj.F3 = F3; obj.F3inv = F3inv;

            [obj.vus, obj.vusH0] = VUS.calcVUS(F1, f2, F3);
            
            % Set full grids for plotting full ROC Surface
            obj.tc1gr = 0:gr_sz:1;
            obj.tc3gr = obj.tc1gr;
            obj.c1gr = obj.F1inv(obj.tc1gr);
            obj.c2gr = obj.F3inv(1-obj.tc3gr);
            tc2 = @(c1, c2) (obj.F2(c2) - obj.F2(c1)).*(c2 > c1);

            [c1_m, c2_m] = meshgrid(obj.c1gr, obj.c2gr);
            sz = size(c1_m);
            Z = tc2(reshape(c1_m, 1, []),reshape(c2_m, 1, []));
            obj.tc2gr = reshape(Z, sz(1), sz(2));
            obj.tc2gr(obj.tc2gr == 0) = -0.01; 
        end   
%-------------------------------------------------------------------------%
% Plot the estimated PDFs of the 3 groups
        function [p1, p2, p3] = plotPDFs(obj, varargin)
            Xrng = [999,-999];
            Finv{1} = obj.F1inv; Finv{2} = obj.F2inv; Finv{3} = obj.F3inv;
            for i = 1:3
                Xrng = [min(Finv{i}(0.001), Xrng(1)),max(Finv{i}(0.999), Xrng(2))];
            end
            x = linspace(Xrng(1),Xrng(2),1./obj.gr_sz);
            hold on;
            p1 = plot(x, obj.f1(x), '-g', 'Color', '#208200', varargin{:});
            p2 = plot(x, obj.f2(x), '-b', 'Color', '#0003E6', varargin{:});
            p3 = plot(x, obj.f3(x), '-r', 'Color', '#F35A0E', varargin{:});
            hold off;  
            p1.Parent.TickLabelInterpreter='latex';
            p2.Parent.TickLabelInterpreter='latex';
            p3.Parent.TickLabelInterpreter='latex';
            xlim([Xrng(1), Xrng(2)])
            title('PDFs');
        end
%-------------------------------------------------------------------------%        
% Plot the ROC Surface        
        function pl = plotROC(obj, varargin)
            pl = surf(obj.tc1gr, obj.tc3gr, obj.tc2gr, varargin{:});
            pl.Parent.TickLabelInterpreter='latex';
            axis([0 1 0 1 0 1]);
            xlabel('$TC_1$','Interpreter', 'Latex'); 
            ylabel('$TC_3$','Interpreter', 'Latex'); 
            zlabel('$TC_2$','Interpreter', 'Latex');
            title('ROC Surface');
            grid off;  box on; caxis([0 1]);
        end
%-------------------------------------------------------------------------%        
% Plot the Uninformative ROC Surface       
        function  pl = plotROCH0(obj, varargin)
            X = [0; 0; 1];
            Y = [0; 1; 0];
            C = [1; 0; 0];
            patch(X,Y,1-X-Y,C,varargin{:});
            pl.Parent.TickLabelInterpreter='latex';
            axis([0 1 0 1 0 1])
            xlabel('$TC_1$','Interpreter', 'Latex'); 
            ylabel('$TC_3$','Interpreter', 'Latex'); 
            zlabel('$TC_2$','Interpreter', 'Latex');
            grid off;  box on; caxis([0 1]);
        end      
        function [p1, p2] = plotROCandROCH0(obj)
            [x_m, y_m] = meshgrid(obj.tc1gr,obj.tc3gr);
            Z = 1 - x_m - y_m; 
            Z(Z < 0) = -0.01;

            % Define a colormap that uses the cool colormap and 
            % the gray colormap and assign it as the Figure's colormap.
            colormap([parula(64);bone(64)]);
            % Produce the two surface plots.
            p1 = surf(obj.tc1gr, obj.tc3gr, obj.tc2gr, 'EdgeColor',...
                'none', 'FaceAlpha', 1);
            hold on;
            p2 = surf(obj.tc1gr, obj.tc3gr, Z, 'EdgeColor','none',...
                'FaceAlpha',0.7, 'FaceLighting', 'none');
            hold off
            view(3)
            m = 64;
            cmin = -0.01;
            cmax = 1;
            C1 = min(m,round((m-1)*(obj.tc2gr-cmin)/(cmax-cmin))+1)./64; 
            C2 = 1+min(m,round((m-1)*(Z-cmin)/(cmax-cmin))+1)./64;
            set(p1,'CData',C1);
            set(p2,'CData',C2);
            p1.Parent.TickLabelInterpreter='latex';
            [min(C1(:)) max(C2(:))];
            caxis([min(C1(:)) max(C2(:))]);
            axis([0 1 0 1 0 1]);
            xlabel('$TC_1$','Interpreter', 'Latex'); 
            ylabel('$TC_3$','Interpreter', 'Latex'); 
            zlabel('$TC_2$','Interpreter', 'Latex');
            grid off;  box on; 
        end
        function  plt = plotOutlineTC2(obj, varargin)
            x = obj.tc1gr;
            plt = plot(x, 1-obj.F3(obj.c1gr), varargin{:});
            axis([0 1 0 1])
            plt.Parent.TickLabelInterpreter='latex';
            xlabel('$TC_1$','Interpreter', 'Latex'); 
            ylabel('$TC_3$','Interpreter', 'Latex'); 
            grid off;
        end
%-------------------------------------------------------------------------%        
% Plot the Outline of the ROC surface on each plane        
        function  [plt1, plt2, plt3] = plotROCOutline(obj, varargin)
            x = obj.tc1gr;
            plt1 = plot3(x, x.*0, 1-obj.F2(obj.c1gr),varargin{:});
            plt2 = plot3(x, 1-obj.F3(obj.c1gr), x.*0, varargin{:});
            plt3 = plot3(x.*0, x, obj.F2(obj.c2gr), varargin{:});
            axis([0 1 0 1 0 1])
            plt1.Parent.TickLabelInterpreter='latex';
            xlabel('$TC_1$','Interpreter', 'Latex'); 
            ylabel('$TC_3$','Interpreter', 'Latex'); 
            zlabel('$TC_2$','Interpreter', 'Latex');
            grid off;  box on; caxis([0 1]);
        end
%-------------------------------------------------------------------------%        
% Plot the Outline of the uninformative ROC surface on each plane        
        function  plotROCOutlineH0(obj, varargin)
            x = [0 1];
            
            plot3(x, x.*0, 1-x,varargin{:})
            plot3(x, 1-x, x.*0, varargin{:})
            plot3(x.*0, x, 1-x, varargin{:})
        end
%-------------------------------------------------------------------------%   
% Plot the ROC with standard formatting options
        function plotROCfmt(obj)
            obj.plotROC('EdgeColor', 'none', 'FaceAlpha', 1)
            hold on
            obj.plotROCOutline('LineWidth', 1, 'Color', [1 1 1].*0.3)
            light('Position',[0 0 2]);
            light('Position',[0 0 0.5],'Style','local');
            light('Position',[0 1 0.5],'Style','local');
            colorbar;
        end
%-------------------------------------------------------------------------%   
% Plot PDFs and ROC Surface Side-by-Side
        function t = plotROCandPDFs(obj, ttl)
            t = tiledlayout(1,2);
            t.TileSpacing = 'compact';
            t.Padding = 'compact';
            nexttile
            obj.plotPDFs('Linewidth', 2)
            title('PDFs', 'FontWeight', 'normal');
            nexttile
            obj.plotROCfmt()
            title('ROC Surface', 'FontWeight', 'normal');
            title(t,ttl, 'FontWeight', 'bold')
            colorbar;
        end
    end
    
    methods(Static)
%-------------------------------------------------------------------------%   
% Calculate VUS and VUS|H0
        function [vus, vusH0] = calcVUS(F1, f2, F3)
           vus = quadgk(@(t) F1(t).*(1-F3(t)).*f2(t), -Inf, Inf, 'MaxIntervalCount', 1000);
           vusH0 = 1/6;
        end
    end
end

