classdef VUSest < VUS
    %UNTITLED2 Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
% Measurement values by group
        X1
        X2
        X3
% Sample Sizes by group
        n1
        n2
        n3
% Estimation Method
        method
% VUS Variance Estimate        
        vvar
% VUS CI        
        vus_ci
% Lambda estimate for box-cox method (will be empty for other methods)        
        lambda
% Transformed Measurements for box-cox method
        X1t
        X2t
        X3t
% Bandwidths for kde methods
        bw1
        bw2
        bw3
        Zstat
        pval
        J
    end
    
    methods
        
        function obj = VUSest(X1, X2, X3, method, outpt, gr_sz)
 % INPUTS:
 %   X1: Observed Measurements for "Healthy" Population
 %   X2: Observed Measurements for "Mildly Diseased" Population
 %   X3: Observed Measurements for "Diseased" Population
 %   method: VUS estimation method
            if nargin < 5
                outpt = true;
            end
            if nargin < 6
                gr_sz=0.0025;
            end
            n1 = length(X1); n2 = length(X2); n3 = length(X3);

%-------------------------------------------------------------------------%
% Normal Method
            if strcmp(method, 'normal')
                [m1, s1, m2, s2, m3, s3,...
                 a, b, c, d,varcov_p,...
                 f1,f2,f3,...
                 F1,F2,F3,...
                 F1inv,F2inv,F3inv] = fitNormals(X1, X2, X3);
% Estimate Derivatives wrt f1 and f2
                %     f1 = (-c*t + d);
                %     f2 = (a*t - b);
            %     df1_dm1 = 0 
                df2_dm1 =-1/s1;
                df1_dm2 = -1/s3; 
                df2_dm2 =1/s1;     
                df1_dm3 = 1/s3; 
                % df2_dm3 =0    
                % df1_ds1 =0 
                df2_ds1 = @(t) (m1 - m2)/s1^2 - (s2*t)/s1^2;    
                df1_ds2 = @(t) -t/s3; 
                df2_ds2 = @(t) t/s1; 
                df1_ds3 = @(t) (m2 - m3)/s3^2 + (s2*t)/s3^2; 
            %     df2_ds3 =0 
% Estimate Derivatives wrt VUS
                dV_ds1 = quadgk(@(t) df2_ds1(t).*normpdf(a.*t-b).*normcdf(-c.*t+d).*normpdf(t), -Inf, Inf); 
                dV_dm1 = quadgk(@(t)    df2_dm1.*normpdf(a.*t-b).*normcdf(-c.*t+d).*normpdf(t), -Inf, Inf);

                dV_ds2 = quadgk(@(t) df2_ds2(t).*normpdf(a.*t-b).*normcdf(-c.*t+d).*normpdf(t), -Inf, Inf)...
                         + quadgk(@(t) df1_ds2(t).*normpdf(-c.*t+d).*normcdf(a.*t-b).*normpdf(t), -Inf, Inf); 
                dV_dm2 = quadgk(@(t)    df2_dm2.*normpdf(a.*t-b).*normcdf(-c.*t+d).*normpdf(t), -Inf, Inf)...
                         + quadgk(@(t)    df1_dm2.*normpdf(-c.*t+d).*normcdf(a.*t-b).*normpdf(t), -Inf, Inf);  

                dV_ds3 = quadgk(@(t) df1_ds3(t).*normpdf(-c.*t+d).*normcdf(a.*t-b).*normpdf(t), -Inf, Inf); 
                dV_dm3 = quadgk(@(t)    df1_dm3.*normpdf(-c.*t+d).*normcdf(a.*t-b).*normpdf(t), -Inf, Inf); 
% Jacobian
                J = [dV_dm1 dV_ds1 dV_dm2 dV_ds2 dV_dm3 dV_ds3];
% Calculate Variance of Vpq Estimate
                vvar = J*varcov_p*J';
% Calculate 95% confidence intervals
                z = norminv(0.975);
                vest = VUS.calcVUS(F1, f2, F3);
                vus_ci = [vest - z*sqrt(vvar), vest + z*sqrt(vvar)];
                X1t = X1; X2t = X2; X3t = X3;
            end % End of Normal method
%-------------------------------------------------------------------------%
% Box-Cox Method
            if strcmp(method, 'boxcox')
                [m1, s1, m2, s2, m3, s3,...
                  a, b, c, d,varcov_p,...
                  X1t, X2t, X3t,lambda,...
                  f1,f2,f3,...
                  F1,F2,F3,...
                  F1inv,F2inv,F3inv] = fitBoxCox(X1, X2, X3);         
% Estimate Derivatives wrt f1 and f2
                df2_dm1 =-1/s1;
                df1_dm2 = -1/s3; 
                df2_dm2 =1/s1;     
                df1_dm3 = 1/s3; 
                df2_ds1 = @(t) (m1 - m2)/s1^2 - (s2*t)/s1^2;    
                df1_ds2 = @(t) -t/s3; 
                df2_ds2 = @(t) t/s1; 
                df1_ds3 = @(t) (m2 - m3)/s3^2 + (s2*t)/s3^2; 
% Estimate Derivatives wrt VUS
                dV_ds1 = quadgk(@(t) df2_ds1(t).*normpdf(a.*t-b).*normcdf(-c.*t+d).*normpdf(t), -Inf, Inf); 
                dV_dm1 = quadgk(@(t) df2_dm1.*normpdf(a.*t-b).*normcdf(-c.*t+d).*normpdf(t), -Inf, Inf);

                dV_ds2 = quadgk(@(t) df2_ds2(t).*normpdf(a.*t-b).*normcdf(-c.*t+d).*normpdf(t), -Inf, Inf)...
                       + quadgk(@(t) df1_ds2(t).*normpdf(-c.*t+d).*normcdf(a.*t-b).*normpdf(t), -Inf, Inf); 
                dV_dm2 = quadgk(@(t) df2_dm2.*normpdf(a.*t-b).*normcdf(-c.*t+d).*normpdf(t), -Inf, Inf)...
                       + quadgk(@(t) df1_dm2.*normpdf(-c.*t+d).*normcdf(a.*t-b).*normpdf(t), -Inf, Inf);  

                dV_ds3 = quadgk(@(t) df1_ds3(t).*normpdf(-c.*t+d).*normcdf(a.*t-b).*normpdf(t), -Inf, Inf); 
                dV_dm3 = quadgk(@(t) df1_dm3.*normpdf(-c.*t+d).*normcdf(a.*t-b).*normpdf(t), -Inf, Inf);
% Jacobian
                J = [dV_dm1 dV_ds1 dV_dm2 dV_ds2 dV_dm3 dV_ds3];
% Estimate Variance of VUS Estimate
                vvar = J*varcov_p*J';
                vest = VUS.calcVUS(F1, f2, F3);
% Calculate 95% confidence interval for VUS Estimate
                z = norminv(0.975);
                vus_ci = [vest - z*sqrt(vvar), vest + z*sqrt(vvar)];
            end % End of Box-Cox Method
%-------------------------------------------------------------------------%
% KDE Methods
            if strcmp(method, 'kde_sym') ||...
                    strcmp(method, 'kde_err_sym') ||...
                    strcmp(method, 'kde_err_prctile') ||...
                    strcmp(method, 'kde_prctile') ||...
                    strcmp(method, 'bc_kde_sym') ||...
                    strcmp(method, 'bc_kde_err_sym') ||...
                    strcmp(method, 'bc_kde_err_prctile') ||...
                    strcmp(method, 'bc_kde_prctile')             
                method_split = split(method, '_');
                
                % Parse method string
                add_boxcox = strcmp(method_split{1}, 'bc');
                prctile = strcmp(method_split{length(method_split)}, 'prctile');
                
                if add_boxcox
                    add_error = strcmp(method_split{3}, 'err');
                else 
                    add_error = strcmp(method_split{2}, 'err');
                end
                if add_boxcox && add_error
                    iqr_1 = iqr(X1);
                    iqr_2 = iqr(X2);
                    iqr_3 = iqr(X3);
                    mn = min([X1; X2; X3]);
                    c = abs(mn) + max([iqr_1; iqr_2; iqr_3]);
                else
                    c = 0;
                end
% Estimate VUS
                [vest, bw1, bw2, bw3, f1,f2,f3,F1,F2,F3,F1inv,...
                  F2inv,F3inv,X1t, X2t, X3t, lambda]  = VUSest.VUSKDEest(X1, X2, X3, add_boxcox, false, 0);
% Set Number of Bootstrap iterations
                B = 400;  
% Initialize Bootstrap sample vector
                vus_kde_boot = zeros(B,1);  
                if outpt
                    fprintf('Generating Bootstrap Estimates\n');    
                end
% START BOOTSTRAP
                    for b = 1:B
                        if outpt
                            fprintf('.')
                            if mod(b, 50) == 0
                                fprintf('\n');
                            end
                        end
% Sample with replacement from each population 
                        X1b = randsample(X1, n1, true);
                        X2b = randsample(X2, n2, true);
                        X3b = randsample(X3, n3, true); 
% Calculate pvus Estimates using Bootstrap samples
                        vus_kde_boot(b,1) = VUSest.VUSKDEest(X1b,X2b,X3b,...
                            add_boxcox, add_error, c);
                    end
% END BOOTSTRAP
    % Estimate Variance of VUS estimate using Bootstrap samples
                vvar = var(vus_kde_boot);
                if prctile
    % Calculate variance and 95% confidence intervals using Percentiles
                    vlower = prctile(vus_kde_boot,2.5);    % Lower Conf. Limit
                    vupper = prctile(vus_kde_boot,97.5);   % Upper Conf. Limit
                else
    % Calculate variance and 95% CIs using normal assumption
                    z = norminv(0.975);
                    vlower = vest - z*sqrt(vvar);
                    vupper = vest + z*sqrt(vvar);
                end
                vus_ci = [vlower, vupper];
% if box-cox transformation is used, transform back to original scale
                if add_boxcox
                    f1or = @(t) f1((t.^lambda-1)./lambda).*t.^(lambda-1);
                    f2or = @(t) f2((t.^lambda-1)./lambda).*t.^(lambda-1);
                    f3or = @(t) f3((t.^lambda-1)./lambda).*t.^(lambda-1);

                    F1invor = @(t) (F1inv(t).*lambda+1).^(1./lambda);
                    F2invor = @(t) (F2inv(t).*lambda+1).^(1./lambda);
                    F3invor = @(t) (F3inv(t).*lambda+1).^(1./lambda);
                else
                    f1or = f1;
                    f2or = f2;
                    f3or = f3;
                    F1invor = F1inv;
                    F2invor = F2inv;
                    F3invor = F3inv;
                end
            end % End of KDE Estimate
            
            obj@VUS(f1,f2,f3,F1,F2,F3,F1inv,F2inv,F3inv,gr_sz);
            
            obj.X1 = X1; obj.n1 = n1;
            obj.X2 = X2; obj.n2 = n2;
            obj.X3 = X3; obj.n3 = n3;
            obj.vvar = vvar;
            obj.vus_ci = vus_ci;
% Set Method            
            obj.method = method;
% Set VUS value under H0 (always 1/6)
            obj.vusH0 = 1/6;
            obj.Zstat = (obj.vus - obj.vusH0)./sqrt(obj.vvar);
            obj.pval = (1 - normcdf(abs(obj.Zstat))).*2;
            if strcmp(method, 'normal') || strcmp(method, 'boxcox')
                obj.J = J;
            end
            if strcmp(method, 'boxcox') || strcmp(method(1:3), 'bc_')
                obj.X1t = X1t; obj.X2t = X2t; obj.X3t = X3t;
                obj.lambda = lambda;
            end
            if strcmp(method(1:4), 'kde') ||...
                    strcmp(method(4:6), 'kde')
                obj.bw1 = bw1; obj.bw2 = bw2; obj.bw3 = bw3;
            end
        end % End of VUSest function
%-------------------------------------------------------------------------%
% Plot the estimated PDFs of the 3 groups
%         function [p1, p2, p3] = plotPDFs(obj, varargin)
%             X =[obj.X1; obj.X1; obj.X3];
%             Xrng = [min(X),max(X)];
%             x = linspace(Xrng(1),Xrng(2),10000);
%             hold on;
%             p1 = plot(x, obj.f1or(x), '-g', 'Color', '#208200', varargin{:});
%             p2 = plot(x, obj.f2or(x), '-b', 'Color', '#0003E6', varargin{:});
%             p3 = plot(x, obj.f3or(x), '-r', 'Color', '#F35A0E', varargin{:});
%             hold off;  
%             p1.Parent.TickLabelInterpreter='latex';
%             p2.Parent.TickLabelInterpreter='latex';
%             p3.Parent.TickLabelInterpreter='latex';
%             xlim([Xrng(1), Xrng(2)])
%             title('PDFs');
%         end
    end   
    methods(Static)
        function [vest, bw1, bw2, bw3, f1,f2,f3,F1,F2,F3,...
              F1inv,F2inv,F3inv, X1t, X2t, X3t, lambda] = VUSKDEest(X1, X2, X3, add_boxcox, add_error, c)
            n1 = length(X1); n2 = length(X2); n3 = length(X3);
            if nargin < 3
                add_boxcox = false;
                add_error = false;
                c = 0;
            end 
            % Box-Cox transform 
            if add_boxcox
                [lambda, X1t, X2t, X3t] = boxcox_roc3(X1, X2, X3); 
            else
                X1t = X1; X2t = X2; X3t = X3; 
                lambda =NaN;
            end
            
            if add_error
                bw1= 0.9*min([std(X1t) iqr(X1t)/1.34])*n1^(-0.2);
                bw2= 0.9*min([std(X2t) iqr(X2t)/1.34])*n2^(-0.2);
                bw3= 0.9*min([std(X3t) iqr(X3t)/1.34])*n3^(-0.2);
                X1t = X1t + normrnd(0, bw1, n1, 1);
                X2t = X2t + normrnd(0, bw2, n2, 1);
                X3t = X3t + normrnd(0, bw3, n3, 1);
                X1t = X1t + c;
                X2t = X2t + c;
                X3t = X3t + c;
            end
            
            [bw1, bw2, bw3,f1,f2,f3,F1,F2,F3,F1inv,F2inv,F3inv]=fitKDE(X1t,X2t,X3t);
            vest = VUS.calcVUS(F1, f2, F3);
        end

    end
end

