%%
clear all
close all
clc

% Set within-group sample sizes
N = [1 1 1].*100
% Randomly Generate Data from three normal distributions
X = [normrnd(10,1,N(1),1);... % Group 1
     normrnd(11,1,N(2),1);... % Group 2
     normrnd(12,1,N(3),1)];   % Group 3
D = [ones(N(1),1); 2.*ones(N(2),1); 3.*ones(N(3),1)];

%% 
close all
clc 
%-------------------------------------------------------------------------%
% Estimates using the Normal Method                                       %
%-------------------------------------------------------------------------%
[vus_est, vus_ci, Z, pval] = estimateVUS(X, D, 'normal'); % VUS
[pvus_est_tc1tc3, pvus_ci_tc1tc3, Z_tc1tc3, pval_tc1tc3] = estimatePVUS(X, D, 'tc1', [0 0.2], 'tc3', [0.6, 0.8], 'normal'); % pVUS(TC1, TC3)
[pvus_est_tc2tc3, pvus_ci_tc2tc3, Z_tc2tc3, pval_tc2tc3] = estimatePVUS(X, D, 'tc3', [0 0.2], 'tc2', [0.6, 0.8], 'normal'); % pVUS(TC2, TC3)
[pvus_est_tc1tc2, pvus_ci_tc1tc2, Z_tc1tc2, pval_tc1tc2] = estimatePVUS(X, D, 'tc1', [0 0.2], 'tc2', [0.6, 0.8], 'normal'); % pVUS(TC1, TC2)

{'Statistic', 'Estimate', '95% CI', 'Z', 'p-value';
 'VUS', round(vus_est,4), round(vus_ci,4), round(Z, 4), pval;
 'pVUS(TC1, TC3)', round(pvus_est_tc1tc3,4), round(pvus_ci_tc1tc3,4), round(Z_tc1tc3, 2), pval_tc1tc3;
 'pVUS(TC2, TC3)', round(pvus_est_tc2tc3,4), round(pvus_ci_tc2tc3,4), round(Z_tc2tc3, 2), pval_tc2tc3;
 'pVUS(TC1, TC2)', round(pvus_est_tc1tc2,4), round(pvus_ci_tc1tc2,4), round(Z_tc1tc2, 2), pval_tc1tc2}



%% 
close all
clc
%-------------------------------------------------------------------------%
% Estimates using the Box-Cox Method                                      %
%-------------------------------------------------------------------------%
[vus_est, vus_ci, Z, pval] = estimateVUS(X, D, 'boxcox'); % VUS
[pvus_est_tc1tc3, pvus_ci_tc1tc3, Z_tc1tc3, pval_tc1tc3] = estimatePVUS(X, D, 'tc1', [0 0.2], 'tc3', [0.6, 0.8], 'boxcox'); % pVUS(TC1, TC3)
[pvus_est_tc2tc3, pvus_ci_tc2tc3, Z_tc2tc3, pval_tc2tc3] = estimatePVUS(X, D, 'tc3', [0 0.2], 'tc2', [0.6, 0.8], 'boxcox'); % pVUS(TC2, TC3)
[pvus_est_tc1tc2, pvus_ci_tc1tc2, Z_tc1tc2, pval_tc1tc2] = estimatePVUS(X, D, 'tc1', [0 0.2], 'tc2', [0.6, 0.8], 'boxcox'); % pVUS(TC1, TC2)

{'Statistic', 'Estimate', '95% CI', 'Z', 'p-value';
 'VUS', round(vus_est,4), round(vus_ci,4), round(Z, 4), pval;
 'pVUS(TC1, TC3)', round(pvus_est_tc1tc3,4), round(pvus_ci_tc1tc3,4), round(Z_tc1tc3, 2), pval_tc1tc3;
 'pVUS(TC2, TC3)', round(pvus_est_tc2tc3,4), round(pvus_ci_tc2tc3,4), round(Z_tc2tc3, 2), pval_tc2tc3;
 'pVUS(TC1, TC2)', round(pvus_est_tc1tc2,4), round(pvus_ci_tc1tc2,4), round(Z_tc1tc2, 2), pval_tc1tc2}

%% 
close all
clc
%-------------------------------------------------------------------------%
% Estimates using the Kernel-Based Method                                 %
%-------------------------------------------------------------------------%
[vus_est, vus_ci, Z, pval] = estimateVUS(X, D, 'bc_kde_sym'); % VUS
[pvus_est_tc1tc3, pvus_ci_tc1tc3, Z_tc1tc3, pval_tc1tc3] = estimatePVUS(X, D, 'tc1', [0.1 0.3], 'tc3',  [0.5, 0.7], 'bc_kde_sym'); % pVUS(TC1, TC3)
[pvus_est_tc2tc3, pvus_ci_tc2tc3, Z_tc2tc3, pval_tc2tc3] = estimatePVUS(X, D, 'tc3',  [0.5, 0.7], 'tc2', [0.1 0.3], 'bc_kde_sym'); % pVUS(TC2, TC3)
[pvus_est_tc1tc2, pvus_ci_tc1tc2, Z_tc1tc2, pval_tc1tc2] = estimatePVUS(X, D, 'tc2', [0.1 0.3], 'tc1', [0.5, 0.7], 'bc_kde_sym'); % pVUS(TC1, TC2)

{'Statistic', 'Estimate', '95% CI', 'Z', 'p-value';
 'VUS', round(vus_est,4), round(vus_ci,4), round(Z, 4), pval;
 'pVUS(TC1, TC3)', round(pvus_est_tc1tc3,4), round(pvus_ci_tc1tc3,4), round(Z_tc1tc3, 2), pval_tc1tc3;
 'pVUS(TC2, TC3)', round(pvus_est_tc2tc3,4), round(pvus_ci_tc2tc3,4), round(Z_tc2tc3, 2), pval_tc2tc3;
 'pVUS(TC1, TC2)', round(pvus_est_tc1tc2,4), round(pvus_ci_tc1tc2,4), round(Z_tc1tc2, 2), pval_tc1tc2}

%%
clear all
close all
clc
%-------------------------------------------------------------------------%
% Examples for Boundaries that lie outside the Feasible Region            %
%-------------------------------------------------------------------------%

N = [50 50 50];
% Randomly Generate Data from three normal distributions
X = [normrnd(11  ,1.1,N(1),1);... % Group 1
     normrnd(11.5,1.2,N(2),1);... % Group 2
     normrnd(12  ,1.3,N(3),1)];   % Group 3
D = [ones(N(1),1); 2.*ones(N(2),1); 3.*ones(N(3),1)];


[pvus_est, pvus_ci] = estimatePVUS(X, D, 'tc3', [0 0.5], 'tc1', [0.6, 0.9], 'normal') % pVUS(TC1, TC3)
[pvus_est, pvus_ci] = estimatePVUS(X, D, 'tc3', [0 0.5], 'tc2', [0.6, 0.9], 'normal') % pVUS(TC2, TC3)
[pvus_est, pvus_ci] = estimatePVUS(X, D, 'tc2', [0 0.5], 'tc1', [0.6, 0.9], 'normal') % pVUS(TC1, TC2)