classdef pVUS_tc2_tc3_est < pVUStc1
    %UNTITLED2 Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
% Measurement values by group
        X1
        X2
        X3
% Sample Sizes by group
        n1
        n2
        n3
% Estimation Method
        method
        pvus_ci
        vvar
        valid_boot_reg
        lambda
%  Transformed Measurements
        X1t
        X2t
        X3t
% Bandwidths for KDE Estimates
        bw1
        bw2
        bw3
        add_boxcox
        add_error
        prc_tile
        pvus_kde_boot
        Zstat
        pval
    end
    
    methods
        
        function obj = pVUS_tc2_tc3_est(X1, X2, X3, p1, p2, q1, q2, method, outpt, gr_sz)
         % INPUTS:
         %   X1: Observed Measurements for "Healthy" Population
         %   X2: Observed Measurements for "Mildly Diseased" Population
         %   X3: Observed Measurements for "Diseased" Population
         %   p1: Lower bound for TC3
         %   p2: Upper bound for TC3
         %   q1: Lower bound for TC2
         %   q2: Upper bound for TC2
         %   method: pVUS estimation method
            if nargin < 9
                outpt = true;
            end
% Calculate Sammple sizes

            n1 = length(X1);
            n2 = length(X2);
            n3 = length(X3);
%-------------------------------------------------------------------------%
% Normal Method
            if strcmp(method, 'normal')
                [m1, s1, m2, s2, m3, s3,...
                    ~, ~, ~, ~,varcov_p,f1,f2,f3,F1,F2,F3,F1inv,F2inv,F3inv]=fitNormals(X1,X2,X3);
                add_boxcox = false;
% Estimate pVUS
                D = pVUS_tc2_tc3_est.getDerivs(m1,s1,m2,s2,m3, s3, p1, p2, q1, q2);
                vest = pVUStc1.calcPVUS(F1, F2, F3, F1inv, F2inv, F3inv, f2, p1, p2, q1, q2);
% Calculate Variance of pVUS Estimate
                vvar = D*varcov_p*D';
% Calculate 95% confidence intervals
                z = norminv(0.975);
                pvus_ci = [vest - z*sqrt(vvar), vest + z*sqrt(vvar)];
                X1t = X1; X2t = X2; X3t = X3;
            end 
%-------------------------------------------------------------------------%
% Box-Cox Method
            if strcmp(method, 'boxcox')
                [m1, s1, m2, s2, m3, s3,~, ~, ~, ~,varcov_p,...
                X1t, X2t, X3t,lambda,f1,f2,f3,F1,F2,F3,F1inv,F2inv,F3inv]=fitBoxCox(X1,X2,X3); 
% Estimate pVUS
                D =  pVUS_tc2_tc3_est.getDerivs(m1,s1,m2,s2,m3, s3, p1, p2, q1, q2);
                vest = pVUStc1.calcPVUS(F1, F2, F3, F1inv, F2inv, F3inv, f2, p1, p2, q1, q2);
% Calculate Variance of pVUS Estimate
                vvar = D*varcov_p*D';
% Calculate 95% confidence intervals
                z = norminv(0.975);
                pvus_ci = [vest - z*sqrt(vvar), vest + z*sqrt(vvar)];
            end
%-------------------------------------------------------------------------%
% KDE Methods
            if strcmp(method, 'kde_sym') ||...
                    strcmp(method, 'kde_err_sym') ||...
                    strcmp(method, 'kde_err_prctile') ||...
                    strcmp(method, 'kde_prctile') ||...
                    strcmp(method, 'bc_kde_sym') ||...
                    strcmp(method, 'bc_kde_err_sym') ||...
                    strcmp(method, 'bc_kde_err_prctile') ||...
                    strcmp(method, 'bc_kde_prctile')
                method_split = split(method, '_');
                
                % Parse method string
                add_boxcox = strcmp(method_split{1}, 'bc');
                prc_tile = strcmp(method_split{length(method_split)}, 'prctile');
                if add_boxcox
                    add_error = strcmp(method_split{3}, 'err');
                else 
                    add_error = strcmp(method_split{2}, 'err');
                end
                if add_boxcox && add_error
                    iqr_1 = iqr(X1);
                    iqr_2 = iqr(X2);
                    iqr_3 = iqr(X3);
                    mn = min([X1; X2; X3]);
                    c = abs(mn) + max([iqr_1; iqr_2; iqr_3]);
                else
                    c = 0;
                end
                [vest, isvalid,bw1, bw2, bw3, f1,f2,f3,F1,F2,F3,...
                    F1inv,F2inv,F3inv, X1t, X2t, X3t, lambda]  = pVUS_tc2_tc3_est.pVUSKDEest(X1, X2, X3, p1, p2, q1, q2, add_boxcox, false, 0);
% Estimate pVUS Variance and Confidence Interval
                if isvalid == 1
                    B = 400;                    % Set Number of Bootstrap iterations
                    pvus_kde_boot = zeros(B,1);  % Initialize Bootstrap sample vector
                    valid_reg_boot = zeros(B,1);
                    if outpt
                        fprintf('Generating Bootstrap Estimates\n');
                    end
% START BOOTSTRAP
                    for b = 1:B
                        if outpt
                            fprintf('.')
                            if mod(b, 50) == 0
                                fprintf('\n');
                            end
                        end
% Sample with replacement from each population 
                        X1b = randsample(X1, n1, true);
                        X2b = randsample(X2, n2, true);
                        X3b = randsample(X3, n3, true); 
% Calculate pvus Estimates using Bootstrap samples
                        [pvus_kde_boot(b,1), valid_reg_boot(b,1)] = pVUSest.pVUSKDEest(X1b, X2b,...
                            X3b, p1, p2, q1, q2, add_boxcox, add_error, c);
                    end
% END BOOTSTRAP
                    vvar = var(pvus_kde_boot);
                    valid_boot_reg = mean(valid_reg_boot); % pct. of valid bootstraps
                    if prc_tile
                        % Calculate variance and 95% confidence intervals
                        % using Percentiles
                        pvus_ci = [prctile(pvus_kde_boot,2.5), prctile(pvus_kde_boot,97.5)];   % Upper Conf. Limit
                    else
                    % Calculate variance and 95% confidence intervals using
                    % variance
                        z = norminv(0.975);
                        pvus_ci = [vest - z*sqrt(vvar), vest + z*sqrt(vvar)];
                    end
                    if add_boxcox
                        F1invor = @(t) (F1inv(t).*lambda+1).^(1./lambda) - c;
                        F2invor = @(t) (F2inv(t).*lambda+1).^(1./lambda) - c;
                        F3invor = @(t) (F3inv(t).*lambda+1).^(1./lambda) - c;

                        f1or = @(t) f1((lambda.*t + 1).^(1/lambda) - c).*(lambda.*t + 1).^(1./lambda - 1);
                        f2or = @(t) f2((lambda.*t + 1).^(1/lambda) - c).*(lambda.*t + 1).^(1./lambda - 1);
                        f3or = @(t) f3((lambda.*t + 1).^(1/lambda) - c).*(lambda.*t + 1).^(1./lambda - 1);
                    else
                        f1or = @(t) f1(t);
                        f2or = @(t) f2(t);
                        f3or = @(t) f3(t);
                        F1invor = @(t) F1inv(t);
                        F2invor = @(t) F2inv(t);
                        F3invor = @(t) F3inv(t);
                    end
                end
            end
            if nargin < 10
                gr_sz = 0.0025;
            end
            obj@pVUStc1(f1,f2,f3,F1,F2,F3,F1inv,F2inv,F3inv,p1,p2,q1,q2,gr_sz);
            obj.method = method;
            obj.X1 = X1; obj.X2 = X2; obj.X3 = X3;
            obj.n1 = n1; obj.n2 = n2; obj.n3 = n3;
            if obj.isvalid
                obj.vvar = vvar;
                obj.pvus_ci = pvus_ci;
                
                if strcmp(method, 'boxcox') || add_boxcox
                    obj.X1t = X1t;
                    obj.X2t = X2t;
                    obj.X3t = X3t;
                    obj.lambda = lambda;
                end
                if strcmp(method(1:3), 'kde') || strcmp(method(4:6), 'kde')
                    obj.bw1 = bw1; obj.bw2 = bw2; obj.bw3 = bw3;
                    obj.valid_boot_reg = valid_boot_reg;
                    obj.add_boxcox = add_boxcox;
                    obj.add_error = add_error;
                    obj.prc_tile = prc_tile;
                    obj.pvus_kde_boot = pvus_kde_boot;
                end
                if obj.isvalidH0 && obj.isvalid
                    obj.Zstat = (obj.pvus - obj.pvusH0)./sqrt(obj.vvar);
                    obj.pval = (1 - normcdf(abs(obj.Zstat))).*2;
                end
            end
        end
    end
    
    methods(Static)
        function D = getDerivs(m1,s1,m2,s2,m3, s3, p1, p2, q1, q2)
                % Calculate Tri-normal Parameters
                a=s2/s1; b=(m1 - m2)./s1;
                c=s2/s3; d=(m3 - m2)./s3;

                Phi = @(t) normcdf(t);
                Phinv = @(t) norminv(t);
                phi = @(t) normpdf(t);
                % set limits of integration
                Bnd1 = Phinv(Phi((d-Phinv(p2))./c) - q2);
                Bnd2 = Phinv(Phi((d-Phinv(p1))./c) - q2);
                Bnd3 = Phinv(Phi((d-Phinv(p2))./c) - q1);
                Bnd4 = Phinv(Phi((d-Phinv(p1))./c) - q1);
                
                if Bnd2 < Bnd3 
                    Q = 0;
                else 
                    Q = 1;
                end
                
                L1 = Bnd1;
                L2 = min(Bnd2, Bnd3);
                L3 = max(Bnd2, Bnd3);
                L4 = Bnd4;
% Estimate Derivatives wrt f1 and f2
                d2_m1 =0;
                d2_m2 =-1/s3; 
                d2_m3 =1/s3;


                d1_m1 =-1/s1; 
                d1_m2 =1/s1;
                d1_m3 =0;
                if Q == 0 
                    A = @(d1, d2_q1, d2_q2) -integral(@(t) phi(d - c.*Phinv(Phi(t) + q2)).*d2_q2.*Phi(a.*t-b).*phi(t), L1, L2);
                    B = @(d1, d2_q1, d2_q2) integral(@(t) (p2 - Phi(d-c.*Phinv(Phi(t)+q2))).*d1.*phi(a.*t - b).*phi(t), L1, L2);
                    C = @(d1, d2_q1, d2_q2) (1-Q).*(p2-p1).*integral(@(t) phi(a.*t - b).*d1.*phi(t), L2, L3);
                    D = @(d1, d2_q1, d2_q2) integral(@(t) phi(d - c.*Phinv(Phi(t) + q1)).*d2_q1.*Phi(a.*t-b).*phi(t), L3, L4);
                    E = @(d1, d2_q1, d2_q2) integral(@(t) (Phi(d - c.*Phinv(Phi(t) + q1)) - p1).*d1.*phi(a.*t-b).*phi(t), L3, L4);
                    dpVUS23_dTheta = @(d1, d2_q1, d2_q2) A(d1, d2_q1, d2_q2)+ B(d1, d2_q1, d2_q2)...
                        + C(d1, d2_q1, d2_q2)+ D(d1, d2_q1, d2_q2)+ E(d1, d2_q1, d2_q2);
                else
                    A = @(d1, d2_q1, d2_q2) -integral(@(t) phi(d - c.*Phinv(Phi(t) + q2)).*d2_q2.*Phi(a.*t-b).*phi(t), L1, L2);
                    B = @(d1, d2_q1, d2_q2) integral(@(t) (p2 - Phi(d-c.*Phinv(Phi(t)+q2))).*d1.*phi(a.*t - b).*phi(t), L1, L2);
                    C = @(d1, d2_q1, d2_q2) Q.*integral(@(t) phi(d-c.*Phinv(Phi(t)+q1)).*d2_q1.*Phi(a.*t-b).*phi(t), L2, L3);
                    D = @(d1, d2_q1, d2_q2) Q.*integral(@(t) Phi(d-c.*Phinv(Phi(t)+q1)).*d1.*phi(a.*t-b).*phi(t), L2, L3);
                    E = @(d1, d2_q1, d2_q2) -Q.*integral(@(t) phi(d-c.*Phinv(Phi(t)+q2)).*d2_q2.*Phi(a.*t-b).*phi(t), L2, L3);
                    F = @(d1, d2_q1, d2_q2) -Q.*integral(@(t) Phi(d-c.*Phinv(Phi(t)+q2)).*d1.*phi(a.*t-b).*phi(t), L2, L3);
                    G = @(d1, d2_q1, d2_q2) integral(@(t) phi(d - c.*Phinv(Phi(t) + q1)).*d2_q1.*Phi(a.*t-b).*phi(t), L3, L4);
                    H = @(d1, d2_q1, d2_q2) integral(@(t) (Phi(d - c.*Phinv(Phi(t) + q1)) - p1).*d1.*phi(a.*t-b).*phi(t), L3, L4);
                    dpVUS23_dTheta = @(d1, d2_q1, d2_q2) A(d1, d2_q1, d2_q2) + B(d1, d2_q1, d2_q2)...
                        + C(d1, d2_q1, d2_q2) + D(d1, d2_q1, d2_q2) + E(d1, d2_q1, d2_q2)...
                        + F(d1, d2_q1, d2_q2) + G(d1, d2_q1, d2_q2) + H(d1, d2_q1, d2_q2);
                end

                % Derivative of pVUS(TC2, TC3) w.r.t mu_1
                dpVUS23_dm1 = dpVUS23_dTheta(d1_m1, d2_m1, d2_m1);
                % Derivative of pVUS(TC2, TC3) w.r.t mu_2
                dpVUS23_dm2 = dpVUS23_dTheta(d1_m2, d2_m2, d2_m2);
                % Derivative of pVUS(TC2, TC3) w.r.t mu_3
                dpVUS23_dm3 = dpVUS23_dTheta(d1_m3, d2_m3, d2_m3);


                % Derivative of pVUS(TC2, TC3) w.r.t sigma_1
                if Q == 0 
                    A = 0;
                    B = integral(@(t) (p2 - Phi(d-c.*Phinv(Phi(t)+q2))).*((m1 - m2)./s1.^2 - (s2.*t)./s1.^2).*phi(a.*t - b).*phi(t), L1, L2);
                    C = (1-Q).*(p2-p1).*integral(@(t) phi(a.*t - b).*((m1 - m2)./s1.^2 - (s2.*t)./s1.^2).*phi(t), L2, L3);
                    D = 0;
                    E = integral(@(t) (Phi(d - c.*Phinv(Phi(t) + q1)) - p1).*((m1 - m2)./s1.^2 - (s2.*t)./s1.^2).*phi(a.*t-b).*phi(t), L3, L4);
                    F = 0; G = 0; H = 0;
                else
                    A = 0;
                    B = integral(@(t) (p2 - Phi(d-c.*Phinv(Phi(t)+q2))).*((m1 - m2)./s1.^2 - (s2.*t)./s1.^2).*phi(a.*t - b).*phi(t), L1, L2);
                    C = 0;
                    D = Q.*integral(@(t) Phi(d-c.*Phinv(Phi(t)+q1)).*((m1 - m2)./s1.^2 - (s2.*t)./s1.^2).*phi(a.*t-b).*phi(t), L2, L3);
                    E = 0;
                    F = -Q.*integral(@(t) Phi(d-c.*Phinv(Phi(t)+q2)).*((m1 - m2)./s1.^2 - (s2.*t)./s1.^2).*phi(a.*t-b).*phi(t), L2, L3);
                    G = 0;
                    H = integral(@(t) (Phi(d - c.*Phinv(Phi(t) + q1)) - p1).*((m1 - m2)./s1.^2 - (s2.*t)./s1.^2).*phi(a.*t-b).*phi(t), L3, L4);
                end
                dpVUS23_ds1 =  A + B + C + D + E + F + G + H;

                % Derivative of pVUS(TC2, TC3) w.r.t sigma_2
                if Q == 0 
                    A =  integral(@(t) phi(d - c.*Phinv(Phi(t) + q2)).*Phinv(q2 + Phi(t))/s3.*Phi(a.*t-b).*phi(t), L1, L2);
                    B =  integral(@(t) (p2 - Phi(d-c.*Phinv(Phi(t)+q2))).*t./s1.*phi(a.*t - b).*phi(t), L1, L2);
                    C =  (1-Q).*(p2-p1).*integral(@(t) phi(a.*t - b).*t./s1.*phi(t), L2, L3);
                    D =  -integral(@(t) phi(d - c.*Phinv(Phi(t) + q1)).*Phinv(q1 + Phi(t))/s3.*Phi(a.*t-b).*phi(t), L3, L4);
                    E =  integral(@(t) (Phi(d - c.*Phinv(Phi(t) + q1)) - p1).*t./s1.*phi(a.*t-b).*phi(t), L3, L4);
                    F = 0; G = 0; H = 0;
                else
                    A =  integral(@(t) phi(d - c.*Phinv(Phi(t) + q2)).*Phinv(q2 + Phi(t))/s3.*Phi(a.*t-b).*phi(t), L1, L2);
                    B =  integral(@(t) (p2 - Phi(d-c.*Phinv(Phi(t)+q2))).*t./s1.*phi(a.*t - b).*phi(t), L1, L2);
                    C =  -Q.*integral(@(t) phi(d-c.*Phinv(Phi(t)+q1)).*Phinv(q1 + Phi(t))/s3.*Phi(a.*t-b).*phi(t), L2, L3);
                    D =  Q.*integral(@(t) Phi(d-c.*Phinv(Phi(t)+q1)).*t./s1.*phi(a.*t-b).*phi(t), L2, L3);
                    E =  Q.*integral(@(t) phi(d-c.*Phinv(Phi(t)+q2)).*Phinv(q2 + Phi(t))/s3.*Phi(a.*t-b).*phi(t), L2, L3);
                    F =  -Q.*integral(@(t) Phi(d-c.*Phinv(Phi(t)+q2)).*t./s1.*phi(a.*t-b).*phi(t), L2, L3);
                    G =  -integral(@(t) phi(d - c.*Phinv(Phi(t) + q1)).*Phinv(q1 + Phi(t))/s3.*Phi(a.*t-b).*phi(t), L3, L4);
                    H =  integral(@(t) (Phi(d - c.*Phinv(Phi(t) + q1)) - p1).*t./s1.*phi(a.*t-b).*phi(t), L3, L4);
                end
                dpVUS23_ds2 =  A + B + C + D + E + F + G + H;

                % Derivative of pVUS(TC2, TC3) w.r.t sigma_3
                if Q == 0 
                    A =  -integral(@(t) phi(d - c.*Phinv(Phi(t) + q2)).*((m2 - m3)./s3.^2 + (s2.*Phinv(q2 + Phi(t)))./s3^2).*Phi(a.*t-b).*phi(t), L1, L2);
                    B =  0;
                    C =  0;
                    D =  integral(@(t) phi(d - c.*Phinv(Phi(t) + q1)).*((m2 - m3)./s3.^2 + (s2.*Phinv(q1 + Phi(t)))./s3^2).*Phi(a.*t-b).*phi(t), L3, L4);
                    E =  0;
                    F = 0; G = 0; H = 0;
                else
                    A =  -integral(@(t) phi(d - c.*Phinv(Phi(t) + q2)).*((m2 - m3)./s3.^2 + (s2.*Phinv(q2 + Phi(t)))./s3^2).*Phi(a.*t-b).*phi(t), L1, L2);
                    B =  0;
                    C =  Q.*integral(@(t) phi(d-c.*Phinv(Phi(t)+q1)).*((m2 - m3)./s3.^2 + (s2.*Phinv(q1 + Phi(t)))./s3^2).*Phi(a.*t-b).*phi(t), L2, L3);
                    D =  0;
                    E =  -Q.*integral(@(t) phi(d-c.*Phinv(Phi(t)+q2)).*((m2 - m3)./s3.^2 + (s2.*Phinv(q2 + Phi(t)))./s3^2).*Phi(a.*t-b).*phi(t), L2, L3);
                    F =  0;
                    G =  integral(@(t) phi(d - c.*Phinv(Phi(t) + q1)).*((m2 - m3)./s3.^2 + (s2.*Phinv(q1 + Phi(t)))./s3^2).*Phi(a.*t-b).*phi(t), L3, L4);
                    H =  0;
                end
                dpVUS23_ds3 =  A + B + C + D + E + F + G + H;
                D = [dpVUS23_dm1 dpVUS23_ds1 dpVUS23_dm2 dpVUS23_ds2 dpVUS23_dm3 dpVUS23_ds3];
        end
%-------------------------------------------------------------------------%
% Estimation of the KDE's for the 3 groups
        function [vest, isValid,bw1, bw2, bw3, f1,f2,f3,F1,F2,F3,...
              F1inv,F2inv,F3inv, X1t, X2t, X3t, lambda] = pVUSKDEest(X1, X2, X3, p1, p2, q1, q2, add_boxcox, add_error, c)
            n1 = length(X1); n2 = length(X2); n3 = length(X3);
            if nargin < 7
                add_boxcox = false;
                add_error = false;
                c = 0;
            end 
            % Box-Cox transform 
            if add_boxcox
                [lambda, X1t, X2t, X3t] = boxcox_roc3(X1, X2, X3); 
            else
                X1t = X1; X2t = X2; X3t = X3;   
                lambda = 0;
            end
            if add_error
                bw1= 0.9*min([std(X1t) iqr(X1t)/1.34])*n1^(-0.2);
                bw2= 0.9*min([std(X2t) iqr(X2t)/1.34])*n2^(-0.2);
                bw3= 0.9*min([std(X3t) iqr(X3t)/1.34])*n3^(-0.2);
                X1t = X1t + normrnd(0, bw1, n1, 1);
                X2t = X2t + normrnd(0, bw2, n2, 1);
                X3t = X3t + normrnd(0, bw3, n3, 1);
                X1t = X1t + c;
                X2t = X2t + c;
                X3t = X3t + c;
            end

            [bw1, bw2, bw3,f1,f2,f3,F1,F2,F3,F1inv,F2inv,F3inv]=fitKDE(X1t,X2t,X3t); 
            [vest, isValid] = pVUStc1.calcPVUS(F1, F2, F3, F1inv, F2inv, F3inv, f2, p1, p2, q1, q2);
        end
    end
end

