classdef pVUStc3 < VUS

    properties
        p1
        p2
        q1
        q2
        
        pvusH0
        isvalidH0
        
        pvus
        isvalid
    end
    
    methods
%-------------------------------------------------------------------------%
% Initialize instance of pVUStc3 Class
        function obj = pVUStc3(f1,f2,f3,F1,F2,F3,F1inv,F2inv,F3inv,p1,p2,q1,q2,gr_sz)
         % INPUTS:
         %   X1: Observed Measurements for "Healthy" Population
         %   X2: Observed Measurements for "Mildly Diseased" Population
         %   X3: Observed Measurements for "Diseased" Population
         %   p1: Lower bound for TC1
         %   p2: Upper bound for TC1
         %   q1: Lower bound for TC2
         %   q2: Upper bound for TC2
         %   method: pVUS estimation method
            if nargin < 20; gr_sz = 0.005; end
            obj@VUS(f1,f2,f3,F1,F2,F3,F1inv,F2inv,F3inv,gr_sz);
            
            obj.p1 = p1;
            obj.p2 = p2;
            obj.q1 = q1;
            obj.q2 = q2;
            
            [obj.pvusH0, obj.isvalidH0] = pVUStc3.calcPVUSH0(p1, p2, q1, q2);
            [obj.pvus, obj.isvalid] = pVUStc3.calcPVUS(F1, F2, F3, F1inv, F2inv, F3inv, f2, p1, p2, q1, q2);
        end
%-------------------------------------------------------------------------%        
% Plot boundaries of pVUS on the x-y plane        
        function plotBounds(obj, varargin)
            plot3([obj.p1 obj.p1],[0 0],[0 1], varargin{:})
            plot3([obj.p2 obj.p2],[0 0],[0 1], varargin{:})
            plot3([0 1],[0 0],[obj.q1 obj.q1], varargin{:})
            plot3([0 1],[0 0],[obj.q2 obj.q2], varargin{:})
        end
%-------------------------------------------------------------------------% 
% Plot outline of floor
        function plotBoundFloor(obj, varargin)
            surf([0 1; 0 1], [0 0], [0 0; 1 1],  varargin{:})  
        end
%-------------------------------------------------------------------------%        
% Plot pVUS region on the ROC Surface      
        function  pl = plotSurf(obj, varargin)
            tc1 = linspace(obj.p1, obj.p2, 1./obj.gr_sz);
            tc2 = linspace(obj.q1, obj.q2, 1./obj.gr_sz);
            tc3 = @(x, z) 1- obj.F3(obj.F2inv(z + obj.F2(obj.F1inv(x))));
            [Xm, Zm] = meshgrid(tc1, tc2);
%             sz = size(Xm);
            Y = tc3(Xm, Zm);
%             Ysurf = reshape(Y, sz(1), sz(2));
            pl = surf(Xm, Y, Zm, varargin{:});
        end
%-------------------------------------------------------------------------%
% Plot the 4 sides of the pVUS
        function [p, r] = plotSurfWithROC(obj, varargin)
            tc3 = @(x, z) 1- obj.F3(obj.F2inv(z + obj.F2(obj.F1inv(x))));
            p = obj.plotSurf(varargin{:});
            x = unique([0:obj.gr_sz:obj.p1, obj.p1:obj.gr_sz:obj.p2, obj.p2:obj.gr_sz:1, 1]);
            z = unique([0:obj.gr_sz:obj.q1, obj.q1:obj.gr_sz:obj.q2, obj.q2:obj.gr_sz:1, 1]);
            [x_m, z_m] = meshgrid(x,z);
            y = tc3(x_m,z_m);
            y(isnan(y)) = -0.01;
            y(z > obj.q1 & z < obj.q2, x > obj.p1 & x < obj.p2) = NaN;
            r = surf(x_m, y, z_m,'EdgeColor', 'none', 'FaceLighting', 'none');
            axis([0 1 0 1 0 1]);
            xlabel('$TC_1$','Interpreter', 'Latex'); 
            ylabel('$TC_3$','Interpreter', 'Latex'); 
            zlabel('$TC_2$','Interpreter', 'Latex');
            grid off;  box on; caxis([0 1]);
        end
%-------------------------------------------------------------------------%        
% Plot pVUS region on the ROC Surface      
        function pl = plotSurfH0(obj, varargin)
            X = [obj.p1; obj.p1; obj.p2; obj.p2];
            Z = [obj.q1; obj.q2; obj.q2; obj.q1];
            Y = 1-X-Z;
            pl = patch(X,Y,Z,'k', varargin{:})
        end
%-------------------------------------------------------------------------%
% Plot the 4 sides of the pVUS
        function [p, r] = plotSurfWithROCH0(obj, varargin)
            p = obj.plotSurfH0(varargin{:});
            X = [1; 0; 0; obj.p1; obj.p1; obj.p2; obj.p2; obj.p1; 0];
            Z = [0; 1; 0; obj.q1; obj.q2; obj.q2; obj.q1; obj.q1; 0];
            Y = 1-Z-X;
            r = patch(X,Y,Z,Z, 'EdgeColor','none','FaceLighting','none', 'FaceAlpha', 0.8);
            r.Parent.TickLabelInterpreter='latex';
            axis([0 1 0 1 0 1]);
            xlabel('$TC_1$','Interpreter', 'Latex'); 
            ylabel('$TC_3$','Interpreter', 'Latex'); 
            zlabel('$TC_2$','Interpreter', 'Latex');
            grid off;  box on; caxis([0 1]);
        end
%-------------------------------------------------------------------------%
% Plot the Outline of the pVUS
        function plotOutline(obj, varargin)
            tc3 = @(x, z) 1- obj.F3(obj.F2inv(z + obj.F2(obj.F1inv(x))));
            gr = 0.01;
            % Line on ROC Surface from (p1,q1) to (p2,q1)
                X1 = [obj.p1:gr:obj.p2, obj.p2];
                Y1 = X1.*0 + obj.q1;
            % Line on ROC Surface from (p1,q2) to (p2,q2) 
                X2 = [obj.p1:gr:obj.p2, obj.p2];
                Y2 = X2.*0 + obj.q2;
            % Line on ROC Surface from (p1,q1) to (p1,q2)
                Y3 = [obj.q1:gr:obj.q2, obj.q2];
                X3 = Y3.*0 + obj.p1;
            % Line on ROC Surface from (p2,q1) to (p2,q2)
                Y4 = [obj.q1:gr:obj.q2, obj.q2];
                X4 = Y4.*0 + obj.p2;
            hold on;
            plot3(X1, tc3(X1, Y1), Y1, varargin{:});
            plot3(X2, tc3(X2, Y2), Y2, varargin{:});
            plot3(X3, tc3(X3, Y3), Y3, varargin{:});
            plot3(X4, tc3(X4, Y4), Y4, varargin{:});
            plot3([obj.p1, obj.p2], [0 0], [obj.q1, obj.q1], varargin{:});
            plot3([obj.p1, obj.p1], [0 0], [obj.q1, obj.q2], varargin{:});
            plot3([obj.p1, obj.p2], [0 0], [obj.q2, obj.q2], varargin{:});
            plot3([obj.p2, obj.p2], [0 0], [obj.q1, obj.q2], varargin{:});
            plot3([obj.p1, obj.p1], [0 tc3(obj.p1, obj.q1)], [obj.q1, obj.q1], varargin{:});
            plot3([obj.p1, obj.p1], [0 tc3(obj.p1, obj.q2)], [obj.q2, obj.q2], varargin{:});
            plot3([obj.p2, obj.p2], [0 tc3(obj.p2, obj.q1)], [obj.q1, obj.q1], varargin{:});
            plot3([obj.p2, obj.p2], [0 tc3(obj.p2, obj.q2)], [obj.q2, obj.q2], varargin{:});
        end
%-------------------------------------------------------------------------%
% Plot the 4 sides of the pVUS
        function plotSides(obj, varargin)
            tc3 = @(x, z) 1- obj.F3(obj.F2inv(z + obj.F2(obj.F1inv(x))));
            p_gr= linspace(obj.p1, obj.p2, 100);
            q_gr= linspace(obj.q1, obj.q2, 100);
            X1 = [obj.p1; obj.p1; p_gr'; obj.p2; obj.p2];
            Z1 = [obj.q1; obj.q1; p_gr'.*0 + obj.q1; obj.q1; obj.q1];
            Y1 = [0; 0; tc3(p_gr',obj.q1); 0; 0];
            Z2 = [obj.q2; obj.q2; p_gr'.*0 + obj.q2; obj.q2; obj.q2];
            Y2 = [0; 0; tc3(p_gr',obj.q2); 0; 0];
            Z3 = [obj.q1; obj.q1; q_gr'; obj.q2; obj.q2];
            X3 = [obj.p1; obj.p1; q_gr'.*0 + obj.p1; obj.p1; obj.p1];
            Y3 = [0; 0; tc3(obj.p1,q_gr'); 0; 0];           
            X4 = [obj.p2; obj.p2; q_gr'.*0 + obj.p2; obj.p2; obj.p2];
            Y4 = [0; 0; tc3(obj.p2,q_gr'); 0; 0];
            patch([X1, X3, X1, X4], [Y1, Y3, Y2, Y4], [Z1,Z3, Z2,Z3], 'k', varargin{:});
            axis([0 1 0 1 0 1])
        end
%-------------------------------------------------------------------------%
% Plot the 4 sides of the pVUS to ROC H0
        function plotSidesToROCH0(obj, varargin)
            tc3 = @(x, z) 1- x - z;
            p_gr= linspace(obj.p1, obj.p2, 100);
            q_gr= linspace(obj.q1, obj.q2, 100);
            X1 = [obj.p1; obj.p1; p_gr'; obj.p2; obj.p2];
            Z1 = [obj.q1; obj.q1; p_gr'.*0 + obj.q1; obj.q1; obj.q1];
            Y1 = [0; 0; tc3(p_gr',obj.q1); 0; 0];
            Z2 = [obj.q2; obj.q2; p_gr'.*0 + obj.q2; obj.q2; obj.q2];
            Y2 = [0; 0; tc3(p_gr',obj.q2); 0; 0];
            Z3 = [obj.q1; obj.q1; q_gr'; obj.q2; obj.q2];
            X3 = [obj.p1; obj.p1; q_gr'.*0 + obj.p1; obj.p1; obj.p1];
            Y3 = [0; 0; tc3(obj.p1,q_gr'); 0; 0];           
            X4 = [obj.p2; obj.p2; q_gr'.*0 + obj.p2; obj.p2; obj.p2];
            Y4 = [0; 0; tc3(obj.p2,q_gr'); 0; 0];
            patch([X1, X3, X1, X4], [Y1, Y3, Y2, Y4], [Z1,Z3, Z2,Z3], 'k', varargin{:});
            axis([0 1 0 1 0 1])
        end
%-------------------------------------------------------------------------%
% Plot the pVUS region on the x-y plane
        function plotFloor(obj, varargin)
            X = [obj.p1; obj.p1; obj.p2; obj.p2];
            Z = [obj.q1; obj.q2; obj.q2; obj.q1];
            Y = X.*0;
            patch(X,Y,Z,'k', varargin{:})
        end
%-------------------------------------------------------------------------%        
% Plot Feasiblity region      
        function pl = plotFeasReg(obj, varargin)
            X = obj.tc1gr;
            Y = X.*0;
            Z = 1 - obj.F2(obj.c1gr) ;
            [Zm, Ym] = meshgrid(Z, Y);
            steps = max(size(X));
            start = zeros(1,steps);

            t = linspace(0,1,steps);
            Xm = (start' * ones(1, steps) + X'*t)';

            pl = surf(Xm, Ym, Zm, varargin{:});
            pl.Parent.TickLabelInterpreter='latex';
            xlabel('$TC_1$','Interpreter', 'Latex'); 
            ylabel('$TC_3$','Interpreter', 'Latex'); 
            zlabel('$TC_2$','Interpreter', 'Latex');
            grid off;  box on; caxis([0 1]);
            axis([0 1 0 1 0 1]);
        end
%-------------------------------------------------------------------------%        
% Plot Feasiblity region for the uninformative ROC Surface     
        function pl = plotFeasRegH0(obj, varargin)
            X = obj.tc1gr;
            Y = X.*0;
            Z = 1 - obj.tc1gr;
            [Zm, Ym] = meshgrid(Z, Y);
            steps = max(size(X));
            start = zeros(1,steps);

            t = linspace(0,1,steps);
            Xm = (start' * ones(1, steps) + X'*t)';

            pl = surf(Xm, Ym, Zm, varargin{:});
            pl.Parent.TickLabelInterpreter='latex';
            xlabel('$TC_1$','Interpreter', 'Latex'); 
            ylabel('$TC_3$','Interpreter', 'Latex'); 
            zlabel('$TC_2$','Interpreter', 'Latex');
            grid off;  box on; caxis([0 1]);
            axis([0 1 0 1 0 1]);
        end
%-------------------------------------------------------------------------%        
% Plot Feasiblity region      
        function pl = plotFeasRegInf(obj, varargin)
            X = obj.tc1gr;
            Y = X.*0;
            Z = 1 - obj.F2(obj.c1gr);
            [Zm, Ym] = meshgrid(Z, Y);
            steps = max(size(X));
            start = zeros(1,steps);

            t = linspace(0,1,steps);
            Xm = (start' * ones(1, steps) + X'*t)';
            Zm(Zm > 1-Xm) = 1 - Xm(Zm > 1-Xm);
            pl = surf(Xm, Ym, Zm, varargin{:});
            pl.Parent.TickLabelInterpreter='latex';
            xlabel('$TC_1$','Interpreter', 'Latex'); 
            ylabel('$TC_3$','Interpreter', 'Latex'); 
            zlabel('$TC_2$','Interpreter', 'Latex');
            grid off;  box on; caxis([0 1]);
            axis([0 1 0 1 0 1]);
        end
%-------------------------------------------------------------------------%        
% Plot Feasiblity region on the x-y plane        
        function  plotFeasRegFlat(obj, varargin)
            x = obj.tc3gr;
            area(x, 1-obj.F3(x), varargin{:});
            xlabel('$TC_1$','Interpreter', 'Latex'); 
            ylabel('$TC_2$','Interpreter', 'Latex'); 
        end    
        function plotFeasRegOutline(obj, varargin)
            x = obj.tc1gr;
            plot3(x, x.*0, 1-obj.F2(obj.c1gr),varargin{:});
            axis([0 1 0 1 0 1])
            xlabel('$TC_1$','Interpreter', 'Latex'); 
            ylabel('$TC_3$','Interpreter', 'Latex'); 
            zlabel('$TC_2$','Interpreter', 'Latex');
            grid off;  box on; caxis([0 1]);
        end
%-------------------------------------------------------------------------%
% Plot one side given values of x, y, and z (helper function for plotSides) 
        function plotSide(obj, X, Y, Z, varargin)
            if length(unique(X)) == 1
                [Zm, Xm] = meshgrid(Z,X);
            else 
                [Xm, Zm] = meshgrid(X,Z);
            end

            steps = max(size(Y));
            start = zeros(1,steps);

            t = linspace(0,1,steps);
            Ym = (start' * ones(1, steps) + Y'*t)';

            surf(Xm, Ym, Zm, varargin{:})
        end
%-------------------------------------------------------------------------%   
% Plot the ROC and pVUS together 
        function plotROCandpVUS(obj)
            obj.plotROC('EdgeColor', 'none', 'FaceAlpha', 1)
            hold on
            obj.plotROCOutline('LineWidth', 1, 'Color', [1 1 1].*0.3)
            obj.plotSides('EdgeColor','none','FaceAlpha',0.6,'FaceColor','#9819E6')
            obj.plotFloor('EdgeColor','none','FaceColor','#420B63')
            obj.plotSurf(true, 'EdgeColor', 'none','FaceColor','#2A0740')
            obj.plotOutline('LineWidth', 1, 'Color', '#2D0845')
            obj.plotBounds('--k', 'Color', 'black') 
            light('Position',[0 0 2]);
            light('Position',[0 0 0.5],'Style','local');
            light('Position',[0 1 0.5],'Style','local');
        end
    end
    
    methods(Static)
%-------------------------------------------------------------------------%
% Check the Validity of the pVUS region
        function v = checkValidity(F1inv, F3inv, p1, p2, q1, q2)
            A = [p1, q1];
            B = [p2, q1];
            C = [p2, q2];
            D = [p1, q2];
            
            % Check if each point of the rectangle is in the Feasibility Region
            inFR = @(pt) (F3inv(1-pt(2)) >= F1inv(pt(1)));
            inFeasReg = [inFR(A), inFR(B), inFR(C), inFR(D)];
            v = (sum(inFeasReg) == 4);
        end
        function [pvusH0, isvalid] = calcPVUSH0(p1, p2, q1, q2)
            if q2 <= 1-p2 || (p2+q2-1) < 1e-10
                pvusH0 = (p2 - p1).*(q2 - q1).*(2-q2-q1-p2-p1)./2;
                isvalid = 1;
            else
                pvusH0 = 0;
                isvalid = 0;
            end
        end    
        function [pvest, isvalid] = calcPVUS(F1, F2, F3, F1inv, F2inv, F3inv, f2, p1, p2, q1, q2)
            if pVUStc3.checkValidity(F1inv, F3inv, p1, p2, q1, q2)
                % Determine all possible boundaries
                Bnd1 = F2inv(F2(F1inv(p1)) + q1);
                Bnd2 = F2inv(F2(F1inv(p2)) + q1);
                Bnd3 = F2inv(F2(F1inv(p1)) + q2);
                Bnd4 = F2inv(F2(F1inv(p2)) + q2);

                % Set Limits of integration;
                L1= Bnd1;
                L2= min(Bnd2, Bnd3);
                L3= max(Bnd2, Bnd3);
                L4 = Bnd4;

                S3 = @(t) 1-F3(t);
                A = @(t) (F1(F2inv(F2(t) - q1)) - p1).*S3(t).*f2(t); 
                if Bnd3 < Bnd2
                    B = @(t) (F1(F2inv(F2(t) - q1)) - F1(F2inv(F2(t) - q2))).*S3(t).*f2(t);
                else
                    B = @(t) (p2 - p1).*S3(t).*f2(t);
                end
                C = @(t) ( p2 - F1(F2inv(F2(t) - q2)) ).*S3(t).*f2(t);
                if L1 == L2; A = @(t) 0; end
                if L3 == L4; C = @(t) 0; end
                pvest = integral(A, L1, L2) + integral(B, L2, L3) + integral(C, L3, L4);
                isvalid = 1;
            else
                pvest = 0;
                isvalid = 0;
            end
        end
    end
end

